@@ -10,6 +10,7 @@ import os
 import logging
 from .Executor import RootExecutor
 from .Executor import GnuPlotExecutor
+from .Executor import RnuPlotExecutor
 # from . import log
 from rich.logging import RichHandler
 from .YamlBlock import YamlFence, CodeFence
@@ -31,122 +32,14 @@ log = logging.getLogger("rich")
 HTML renderer for mistletoe with ROOT code execution and asset injection.
 """
 
-core_css = """
-
-html {
-        font-family: 'Roboto', sans-serif;
-    }
-
-    .content {
-            max-width: 95%;
-            margin: auto;
-        }
-    @media (min-width:1200px) {
-        .content {
-            max-width: 75%;
-        }
-    }
-
-    @media (min-width:1900px) {
-        .content {
-            max-width: 60%;
-        }
-    }
-
-    .png {
-        display: inline-block;
-        margin-left: auto;
-        margin-right: auto;
-        max-width: 100%;
-    }
-
-    pre {
-        font-size: 0.9em!important;
-        line-height: 0.9!important;
-    }
-
-
-    .svg {
-        width: 100%;
-    }
-
-div.root-block pre {
-    margin-top: 0px!important;
-    /* margin-bottom: 0px!important; */
-    border-left: 5px solid grey;
-    border-right: 5px solid grey;
-    border-bottom: 2px solid grey;
-    border-top: 2px solid grey;
-}
-
-div.root-block-green pre {
-    margin-top: 0px!important;
-    margin-bottom: 0px!important;
-
-    border-left: 5px solid #64dd17;
-    border-right: 5px solid #64dd17;
-    /* border-bottom: 2px solid #64dd17; */
-    /* border-top: 2px solid #64dd17; */
-}
-"""
-
 pygments_extra_css = """
 pre {
-  line-height: 1.5em !important;
-  font-size: 1.1em !important;
+    line-height: 1.1em!important;
+    text-size: 1.1em!important;
+    font-family: 'Fira Code'
 }
 """
 
-
-doct = """
-<!DOCTYPE html>
-<html lang="en">
-  <head>
-    <meta charset="UTF-8">
-    <meta name="description" content="{description}">
-    <meta name="keywords" content="{keywords}">
-    <meta name="author" content="{author}">
-    <meta name="viewport" content="width=device-width, initial-scale=1.0">
-    <meta http-equiv="X-UA-Compatible" content="ie=edge">
-    <title>{title}</title>
-
-
-    <link rel="preconnect" href="https://fonts.googleapis.com">
-    <link rel="preconnect" href="https://fonts.gstatic.com" crossorigin>
-    <link href="https://fonts.googleapis.com/css2?family=Roboto" rel="stylesheet"> 
-    <script src="https://polyfill.io/v3/polyfill.min.js?features=es6"></script>
-    <script id="MathJax-script" async src="https://cdn.jsdelivr.net/npm/mathjax@3.0.1/es5/tex-mml-chtml.js"></script>
-    
-
-
-    <!-- PRISM JS -->
-        <!-- PRISM JS : core -->
-        <script src="https://cdnjs.cloudflare.com/ajax/libs/prism/1.26.0/prism.min.js" integrity="sha512-pSVqGtpGygQlhN8ZTHXx1kqkjQr30eM+S6OoSzhHGTjh6DKdfy7WZlo1DNO9bhtM0Imf6xNLznZ7iVC2YUMwJQ==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
-        <!-- <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/prism/1.26.0/themes/prism-okaidia.min.css" integrity="sha512-mIs9kKbaw6JZFfSuo+MovjU+Ntggfoj8RwAmJbVXQ5mkAX5LlgETQEweFPI18humSPHymTb5iikEOKWF7I8ncQ==" crossorigin="anonymous" referrerpolicy="no-referrer" /> -->
-
-
-        <!-- PRISM JS : autoloader JS -->
-    <script src="https://cdnjs.cloudflare.com/ajax/libs/prism/1.26.0/plugins/autoloader/prism-autoloader.min.js" integrity="sha512-GP4x8UWxWyh4BMbyJGOGneiTbkrWEF5izsVJByzVLodP8CuJH/n936+yQDMJJrOPUHLgyPbLiGw2rXmdvGdXHA==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
-
-        <!-- PRISM JS : inline -->
-    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/prism/1.26.0/plugins/inline-color/prism-inline-color.min.css" integrity="sha512-jPGdTBr51+zDG6sY0smU+6rV19GOIN9RXAdVT8Gyvb55dToNJwq2n9SgCa764+z0xMuGA3/idik1tkQQhmALSA==" crossorigin="anonymous" referrerpolicy="no-referrer" />
-    <script src="https://cdnjs.cloudflare.com/ajax/libs/prism/1.26.0/plugins/inline-color/prism-inline-color.min.js" integrity="sha512-U2u7V7F0Yk6Cw3LrZMYBDKQ+FbGigq+Z0JhHI04iKjtNXZUm4RdHsJ4xVbJLTiIFhNZ/5/3M12I1wXQtvxXB/w==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
-
-        <!-- PRISM JS : line-numbers -->
-    <script src="https://cdnjs.cloudflare.com/ajax/libs/prism/1.26.0/plugins/line-numbers/prism-line-numbers.min.js" integrity="sha512-dubtf8xMHSQlExGRQ5R7toxHLgSDZ0K7AunqPWHXmJQ8XyVIG19S1T95gBxlAeGOK02P4Da2RTnQz0Za0H0ebQ==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
-    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/prism/1.26.0/plugins/line-numbers/prism-line-numbers.min.css" integrity="sha512-cbQXwDFK7lj2Fqfkuxbo5iD1dSbLlJGXGpfTDqbggqjHJeyzx88I3rfwjS38WJag/ihH7lzuGlGHpDBymLirZQ==" crossorigin="anonymous" referrerpolicy="no-referrer" />
-
-    <style>
-    {css}
-    </style>
-  </head>
-  <body>
-    <div class="content" >
-    {body}
-    </div>
-  </body>
-</html>"""
-
 codetemplate = '<pre class="languag-{lang}"><code class="language-{lang}">{inner}</code></pre>'
 divtemplate = '<div class="output" >' + codetemplate + '</div>'
 imgtemplate = '<img src="{src}" class="{ext}  {ucls}"/>'
@@ -182,6 +75,8 @@ class RootHtmlRenderer(HTMLRenderer, RootExecutor):
 
         # additional executors
         self.gnuplot = GnuPlotExecutor()
+        self.rnuplot = RnuPlotExecutor()
+        self.highlighter = self.yaml.get("highlight", "prismjs").lower()
     
     def set( self, **kwargs ) :
         if "embed" in kwargs :
@@ -253,6 +148,13 @@ class RootHtmlRenderer(HTMLRenderer, RootExecutor):
             return ""
         return super().render_block_code(token)
 
+    def hpygments( self, token ):
+        code = token.children[0].content
+        lexer = get_lexer_by_name( token.language)
+        formatter = HtmlFormatter( style=self.yaml.get( "pygments-theme", "dracula" ))
+        return highlight(code, lexer, formatter)
+        
+
     def process_yaml( self, yaml ):
         self.yaml = yaml
         self.title = yaml.get( "title", self.title )
@@ -268,7 +170,10 @@ class RootHtmlRenderer(HTMLRenderer, RootExecutor):
     def process_gnuplot( self, token ):
         log.info( "gnuplot" )
         token.language = "python" # just for output syntax highlighting
-        code_block =  super().render_block_code(token)
+        if self.highlighter == "pygments":
+            code_block = self.hpygments( token )
+        else:
+            code_block =  super().render_block_code(token)
         code =token.children[0].content
         imgs = self.gnuplot.find_output( code )
         img_class = ""
@@ -283,6 +188,28 @@ class RootHtmlRenderer(HTMLRenderer, RootExecutor):
         imgout += '</div>'
 
         return code_block + self.divWrap(imgout, "gnuplot-block", "gnuplot-output-block-%d" % (self.blockid - 1) )
+
+    def process_rnuplot( self, token ):
+        log.info( "rnuplot" )
+        token.language = "python" # just for output syntax highlighting
+        if self.highlighter == "pygments":
+            code_block = self.hpygments( token )
+        else:
+            code_block =  super().render_block_code(token)
+        code =token.children[0].content
+        imgs = self.rnuplot.find_output( code )
+        img_class = ""
+
+        log.info( "Executing rnuplot" )
+        self.rnuplot.run( code )
+
+        # inject images
+        imgout = '<div id="{id}" class="root-images" style="text-align: center;">'.format( id="root-images-%d" % (self.blockid))
+        for i in imgs:
+            imgout += self.process_image_output( i, ucls=img_class )
+        imgout += '</div>'
+
+        return code_block + self.divWrap(imgout, "gnuplot-block", "gnuplot-output-block-%d" % (self.blockid - 1) )
     
 
     """
@@ -306,7 +233,10 @@ class RootHtmlRenderer(HTMLRenderer, RootExecutor):
         return  not (options.get( n, "" ).lower() == 'false' or options.get( n, "" ) == '0')
 
     def process_cpp( self, token ):
-        code_block =  super().render_block_code(token)
+        if self.highlighter == "pygments":
+            code_block = self.hpygments( token )
+        else:
+            code_block =  super().render_block_code(token)
         code = token.children[0].content
         
         log.info( "exec: %r" % self.optionAsBool( token.options, "exec", True ) )
@@ -418,6 +348,9 @@ class RootHtmlRenderer(HTMLRenderer, RootExecutor):
 
         if "gnuplot" == token.language or token.options.get( "exec", "" ) == "gnuplot":
             return self.process_gnuplot( token )
+        
+        if "rnuplot" == token.language or token.options.get( "exec", "" ) == "rnuplot":
+            return self.process_rnuplot( token )
 
         if "cpp" == self.escape_html(token.language) or token.options.get( "exec", "" ) == "cpp":
             return self.process_cpp( token )
@@ -448,7 +381,13 @@ class RootHtmlRenderer(HTMLRenderer, RootExecutor):
 
     def render_document(self, token):
         self.footnotes.update(token.footnotes)
+        mydir = os.path.dirname(os.path.abspath(__file__))
         inner = '\n'.join([self.render(child) for child in token.children])
+        try : 
+            with open( os.path.join(mydir, "data/css/core.css")) as f:
+                core_css = ''.join(f.readlines())
+        except Exception as e:
+            log.error(e)
         css = core_css
         if self.yaml.get("highlight", "prismjs").lower() == "prismjs" :
             try:
@@ -458,7 +397,12 @@ class RootHtmlRenderer(HTMLRenderer, RootExecutor):
         
         if self.yaml.get("highlight", "prismjs").lower() == "pygments":
             css = css + HtmlFormatter(style=self.yaml.get( "pygments-theme", "dracula" )).get_style_defs('.highlight') + pygments_extra_css
-        return doct.format( 
+        html = ""
+        try :
+            with open( os.path.join(mydir, "data/template.html")) as f:
+                html = ''.join(f.readlines())
+
+            output =  html.format( 
                 title=self.title, 
                 description=self.description,
                 author=self.author,
@@ -466,3 +410,7 @@ class RootHtmlRenderer(HTMLRenderer, RootExecutor):
                 css=css, 
                 body= '\n\n{}\n'.format(inner) if inner else '' 
             )
+            return output
+        except Exception as e:
+            log.error(e)
+        return inner
