@@ -36,10 +36,10 @@ Entry point for RootMd
 """
 parser = argparse.ArgumentParser(description='Convert Markdown with inline c++ code to ROOT output.', prog="rootmd")
 parser.add_argument(
-    'input', help='input Markdown file to execute and convert'
+    'input', nargs="?", help='input Markdown file to execute and convert', default=""
 )
 parser.add_argument(
-    '--output', help='output filename  default <input>.<ext> where <ext> is determined by the chosen format, default html', default=""
+    '--output', help='output filename  default <input>.<ext> where <ext> is determined by the chosen format, default html, output can include {input} to allow substitution of input filename, or {path} for full path, or {basepath} for path to input file, or {ext} for default output extension for format specified. TODO', default=""
 )
 parser.add_argument( 
     '--format', help='output format', default="html", 
@@ -67,8 +67,18 @@ parser.add_argument(
     '--no-exec', 
     help='Do not execute any code blocks, just process file (useful for viewing and testing conversion)', action='store_true', default=False)
 
+parser.add_argument( 
+    '--html-template', 
+    help='Template HTML file should include {js}, {css}, {content} ... TODO', default="")
+parser.add_argument( 
+    '--css', 
+    help='CSS used for the HTML output, overrides default ... TODO', default="")
+
 args = parser.parse_args()
 
+if "input" not in args and "watch" not in args :
+    log.error( "Must provide one of [\"[input]\" or \"--watch\" to specify input files ]" )
+    exit()
 
 EXTENSIONS = {
     "html"     : ".html",
